@@ -8,7 +8,7 @@ function(diffs,histbreaks=50000,mzmin=0,mzmax=0.5,cutcount=180,plotit=TRUE){
    masstol2<-c((his[[1]][2]-his[[1]][1])/2);
    his<-his$mids[his$counts>cutcount & his$mids<=mzmax & his$mids>=mzmin];
    #his<-his[his>(0+masstol2)]; # not to search peak = peak !
-   that<-data.frame(as.character(rep(1:length(his))),his,rep(0,length(his)),rep(0,length(his)),
+   that<-data.frame(as.character(rep(1:length(his))),his,rep(0,length(his)),rep(0,length(his),stringsAsFactors=FALSE),
    rep(1,length(his)),rep(0,length(his)),rep(1,length(his)));
    names(that)<-c("name","dmass","dabund","how_often","#atoms","/C","z");
    elements<-as.character(rep(1:length(his)));
